package Linux::MountPoint;

use 5.010;
use strict;
use warnings;
use Carp;

=encoding utf8

=head1 NAME

Linux::MountPoints -- List all mountpints

=head1 VERSION

Version 0.01

=cut
our $VERSION = '0.01';

=head1 SYNOPSIS

 use Linux::MountPoint;

 print_out(Linux::MountPoint->root(),'');

 sub print_out
 {
   my $point=shift;
   my $in=shift // '';
   print $in.'Mountpoint: '.$point->mount_point()."\n";
   print $in.'FS Type   : '.$point->fstype()."\n";
   print $in.'Source    : '.$point->source()."\n";
   print $in."--------\n";
   print_out($_,$in.'  ') for(sort{$a->mount_point() cmp $b->mount_point()}$point->childs());
 }

=head1 DESCRIPTION

Information about the MountPoints

=head1 AUTHOR

ToPeG

=head1 PACKAGE VARIABLES

=over 4

=item $minfos_file

predefined: /proc/<PID>/mountinfo

=cut

our $minfos_file="/proc/$$/mountinfo";
my @infos;

=back

=head1 METHODS

=over 4

=item list()

returns a list of MountPoint-Objects
on error it retuns aln empty list.

=cut

sub list
{
  my $class=__PACKAGE__;
  $class=shift() if($_[0]);
  @infos=__parse_mountinfo($minfos_file);
  return map{$class->_new($_)}@infos;
}

sub _new
{
  my $class=shift;
  my $data=shift // {};
  return bless({ %$data },ref($class) || $class);
}

sub _get
{
  my $self=shift;
  my $name=shift;
  return $self->{$name} if(exists($self->{$name}));
  return '';
}

#-----------------------------------------------------------------------

=item mount_options()

returns a list of the mount Options

=cut
sub mount_options { shift->{mount_options}; }

=item super_options()

returns a list of the super Options

=cut
sub super_options { shift->{super_options}; }

=item optional_fields()

returns a hash of the optional Fields
Options of the Kernelmodul

=cut
sub optional_fields { shift->{optional_fields}; }

=item mount_point()

returns the mountpoint

=cut
sub mount_point { shift->{mount_point}; }

=item fstype()

returns the Filesystem Type

=cut
sub fstype { shift->{fstype}; }

=item device()

returns the Device.
If it's a vitual Filesystems it's the name of the kernelmodule.

=cut
sub device { shift->{mount_source}; }

=item mount_id()

returns an unique ID

=cut
sub mount_id { shift->{mount_id}; }

=item parent_id()

returns the ID of the parent MountPoint

=cut
sub parent_id { shift->{parent_id}; }

=item minor()

returns the Kernel minor ID

=cut
sub minor { shift->{minor}; }

=item major()

returns the Kernel major ID

=cut
sub major { shift->{major}; }

=item is_virtual()

returns true if the filesystem is virtual

=cut
sub is_virtual
{
  my $self=shift;
  return 0 if($self->fstype() eq $self->device());
  return 1;
}

=item is_bind()

returns true if the Mpuntpoint is moutet with the "bind" option

=cut
sub is_bind
{
  my $self=shift;
  return 1 if($self->{root} ne '/');
  return 0;
}

=item has_parent()

returns true if the Mountpoint have a parent

=cut
sub has_parent
{
  $_[0]->parent_id() == $_->{mount_id} and return 1 for(@infos);
  return 0;
}

=item source()

returns the real source of the Mountpoint.
If it's mounted with "bind" the source is the path to the mount_source.
Otherwise i'ts identical to device().
=cut
sub source
{
  my $self=shift;
  if($self->is_bind())
  {
    for(@infos)
    {
      next unless($self->minor() == $_->{minor} && $self->major() == $_->{major});
      next if($self->{mount_point} eq $_->{mount_point});
      return $_->{mount_point}.$self->{root};
    }
  }
  return $self->device();
}

=item parent()

returns the Parent Mountpoint Object

=cut
sub parent
{
  for(@infos)
  { return $_[0]->_new($_) if($_[0]->parent_id() == $_->{mount_id}) }
  return undef;
}

=item childs()

returns a list of Child Mountpoint Objects

=cut
sub childs
{
  my $self=shift;
  my @ret;
  return () unless($self->mount_id());
  for my $mount (@infos)
  {
    next unless($mount->{parent_id});
    next if($mount->{parent_id} != $self->mount_id());
    push(@ret,$self->_new($mount));
  }
  return @ret;
}

=item flatten()

returns a list of all Child Mountpoint Objects

=cut
sub flatten
{
  my $self=shift;
  my @ret;
  push(@ret,$_,$_->flatten()) for($self->childs());
  return @ret;
}

=item root()

returns the root-filesystem Mountpoint

=cut
sub root
{
  my $self=shift;
  unless(ref($self))
  {
    my @l=$self->list();
    return undef unless (@l);
    $self=$l[0];
  }
  $self=$self->parent() while($self->has_parent());
  return $self;
}
#-----------------------------------------------------------------------

sub __parse_mountinfo
{
  my $file=shift // $minfos_file;
  open(my $fh, '<', $file) or croak "Unable to open '$file': $!";
  my @ret;
  while(<$fh>)
  {
    my @line=split(/\s+/,$_);
    my %info;

    $info{mount_id}      = shift(@line);
    $info{parent_id}     = shift(@line);

    my @m=split(/:/,shift(@line));
    $info{major}         = shift(@m);
    $info{minor}         = shift(@m);

    $info{root}          = shift(@line);
    $info{mount_point}   = shift(@line);
    $info{mount_options} = [split(/,/,shift(@line))];

    $info{optional_fields}={};
    while(my $val=shift(@line))
    {
      last if($val eq '-');
      my ($k,$v)=split(/:/,$val);
      $info{optional_fields}->{$k}=$v;
    }

    $info{fstype}        = shift(@line);
    $info{mount_source}  = shift(@line);
    $info{super_options} = [split(/,/,shift(@line))];
    push(@ret,\%info);
  }
  close($fh);
  return @ret;
}

=back

=head1 AUTHOR

Tobias Grönhagen, C<< <linux_mountpoint at topeg.de> >>

=head1 BUGS

Please report any bugs or feature requests to C<bug-linux-mountpoint at rt.cpan.org>, or through
the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Linux-MountPoint>.  I will be notified, and then you'll
automatically be notified of progress on your bug as I make changes.




=head1 SUPPORT

You can find documentation for this module with the perldoc command.

    perldoc Linux::MountPoint


You can also look for information at:

=over 4

=item * RT: CPAN's request tracker (report bugs here)

L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Linux-MountPoint>

=item * AnnoCPAN: Annotated CPAN documentation

L<http://annocpan.org/dist/Linux-MountPoint>

=item * CPAN Ratings

L<http://cpanratings.perl.org/d/Linux-MountPoint>

=item * Search CPAN

L<http://search.cpan.org/dist/Linux-MountPoint/>

=back

=head1 COPYRIGHT

Copyright 2012 Tobias Grönhagen. All rights reserved.

=head1 LICENSE

This program is free software; you can redistribute it and/or modify it
under the terms of either: the GNU General Public License (GPL) version 2 as published
by the Free Software Foundation; or the Artistic License.

See http://dev.perl.org/licenses/ for more information.

=head1 DISCLAIMER

THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS
OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE.

Use of this software in any way or in any form, source or binary,
is not allowed in any country which prohibits disclaimers of any
implied warranties of merchantability or fitness for a particular
purpose or any disclaimers of a similar nature.

IN NO EVENT SHALL I BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
SPECIAL, INCIDENTAL,  OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
USE OF THIS SOFTWARE AND ITS DOCUMENTATION (INCLUDING, BUT NOT
LIMITED TO, LOST PROFITS) EVEN IF I HAVE BEEN ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE

=cut
1;